#++ ---------------- sympower.mu ---------------------

Author: Winfried Fakler,  University of Karlsruhe
Date Created: 24 April    1996
Last Update :  8 November 1996

Description:
symmetricPower( Ly, y(x), m ) computes the m-th symmetricPower of the 
linear homogeneous differential equation Ly. This is the lowest order
linear homogeneous differential equation whose solution space consists 
exactly of all possible m-th powerproducts of solutions of Ly.

Functions:

 - used Parameter:
    ++ Ly, y(x):DOM_EXPR
    ++ y, x    :DOM_IDENT
    ++ n, m    :DOM_INT
    ++ Ly   = ordinary linear differential equation
    ++ y(x) = the operator (function) of Ly  
    ++ y    = name of the operator of Ly
    ++ x    = dependant variable of the operator y
    ++ n    = order of Ly
    ++ m    = exponent of the symmetric power of Ly

 - ode::symmetricPower(Ly, y(x), m)
    ++ returns the m-th symmetricPower of an ordinary homogeneous
    ++ linear differential equation Ly.
    ++ The given arguments are tested.
 - ode::symPower(Ly, y, x, n, m)
    ++ returns the m-th symmetricPower of an ordinary
    ++ linear differential equation Ly.
    ++ NOTE: the reductum of Ly needs to be non-constant! Otherwise the result
    ++       maybe wrong!
 - ode::symPowerOrd2(Ly, y, x, m)
    ++ returns the m-th symmetricPower of a second order ordinary
    ++ linear differential equation Ly.
    ++ NOTE: the reductum of Ly needs to be non-constant! Otherwise the result
    ++       maybe wrong!

See: Singer, M. F. (1985). Solving homogeneous linear differential 
     equations in terms of second order linear differential equations. 
     Amer. J. Math. 107

Example:

>> eq := diff(y(x),x,x)+1/x*diff(y(x),x)-1/(4*x^2)*y(x):
>> ode::symmetricPower(eq, y(x), 2);
 
                 3 diff(y(x), x, x)
                 ------------------ + diff(y(x), x, x, x)
                         x
Hurwitz-equation:
>> eq:=x^2*(x-1)^2*diff(y(x),x,x,x)+(7*x-4)*x*(x-1)*diff(y(x),x,x)+
 (72/7*(x^2-x)-20/9*(x-1)+3/4*x)*diff(y(x),x)+(72*11/7^3*(x-1)+5/8+2/63)*y(x):
>> ode::symmetricPower(eq, y(x), 2);
++#

sysassign(ode::symmetricPower, proc(eq, z, m) 
  local vars;
  begin
    eq := ode::isLODE(eq, z, "Hlode");
    case nops(eq) 
      of 1 do return(error("not an ordinary homogeneous linear differential ".
                           "equation"));
      of 4 do if eq[4] < 1 then
                 return(error("only defined for positive order equations"))
              end_if;
      otherwise
        if domtype(m) <> DOM_INT or m < 1
          then return(error("only defined for positive integers"))
        end_if
    end_case;
    if eq[4]=2 
      then eq[4] := NIL; ode::symPowerOrd2(eq, m)
      else ode::symPower(eq, m) end_if
  end_proc):

sysassign(ode::symPower, proc(eq, z, x, n, m) 
  local t0, c, Y, i, j, peq, 
        ansatz, dimBasis, newDimBasis, diffvars, solset, term;
  begin
    t0 := time();
    z := z(x);
    Y := z^m;
    peq := poly(eq, [diff(z, x$n)]);
    term := expand(- tcoeff(peq)/lcoeff(peq));
    ansatz := c[0]*Y;
    diffvars := [diff(z, x$j) $ j=0..n-1];
    for i from 1 to n-1 do
      Y := diff(Y, x);
      ansatz := ansatz + c[i]*Y;
    end_for;    
    i := n-1;
    newDimBasis := nterms(ansatz, diffvars);
    repeat
      i := i+1;
      dimBasis := newDimBasis;
      userinfo(5, "order is now: ", i);
      Y := expand(subs(diff(Y, x), diff(z, x$n)=term));
      ansatz := expand(ansatz + c[i]*Y);
      newDimBasis := nterms(ansatz, diffvars);
    until dimBasis = newDimBasis end_repeat; 
    repeat
      userinfo(5, "...solve ansatz");
      [coeff(poly(subs(ansatz, c[i]=1), diffvars))];
      solset := linsolve({op(%, j)=0 $j=1..dimBasis}, {c[j] $j=0..i-1});
      if solset <> FAIL and solset <> null() then
        userinfo(10, "Time: ", time()-t0, "msec");
        [op({c[i]=1} union solset, j) $j=1..i+1];
        return( subs( _plus(c[j]*diff(z, x$j) $j=0..i), %) ) 
      end_if;
      i := i+1;
      userinfo(5, "order is now: ", i);
      Y := expand(subs(diff(Y, x), diff(z, x$n)=term));
      ansatz :=  expand(ansatz + c[i]*Y) 
    until FALSE end_repeat;
  end_proc): 

sysassign(ode::symPowerOrd2, proc(eq, z, x, m) 
  local t0, c, Y, i, j, dxn, sys, unk, ind, e, eqn, sols, 
        ansatz, dimBasis, newDimBasis, diffvars, solset, term;
  begin
    t0 := time();
    z := z(x);
    Y := z^m;
    dxn := diff(z, x$2);
    diffvars := [diff(z, x$j) $hold(j)=0..2]; 
    term := expand(- tcoeff(eq, [dxn])/lcoeff(eq, [dxn]));
    c[0] := genident("c");
    ansatz := c[0]*Y;
    Y := diff(Y, x);
    c[1] := genident("c");
    ansatz := ansatz + c[1]*Y;  
    i := 1; 
    newDimBasis := nterms(ansatz, diffvars);
    repeat
      i := i+1;
      dimBasis := newDimBasis;
      userinfo(5, "order is now: ", i);
      Y := expand(subs(diff(Y, x), dxn=term));
      c[i] := genident("c");
      ansatz := ansatz + c[i]*Y;
      newDimBasis := nterms(ansatz, diffvars);
    until dimBasis = newDimBasis end_repeat; 
    repeat
      userinfo(5, "...solve ansatz");
      sys := {coeff(poly(ansatz, diffvars))};
      unk := {c[j] $hold(j)=0..i-1};
      # the following lines avoid an expensive call to linsolve #
      sols := null();
      repeat
        e:=FALSE; 
        for eqn in sys do
          ind := indets(eqn) intersect unk;
          if nops(ind)=1 then
            ind := op(ind);
            if degree(eqn,[ind])=1 then
              sys := sys minus {eqn}; unk := unk minus {ind};
              # normal is the expensive call #
              #eqn := ind=normal(-coeff(eqn,[ind],0)/coeff(eqn,[ind],1));#
              eqn := ind=expand(-coeff(eqn,[ind],0)/coeff(eqn,[ind],1));
              sols := sols,eqn; sys := subs(sys,eqn); 
              e := TRUE; break;
            end_if
          end_if
        end_for;
      until e=FALSE end_repeat;
      sys:=sys minus {0};
      if sys<>{} then
        solset := linsolve(sys,unk);
        #print("linsolve");#
       else solset := {} end_if;
      if solset <> FAIL and solset <> null() then
        solset := {sols} union solset;
        if has(solset, c[i])  
         then solset := subs(solset, c[i]=1) union {c[i]=1}
         else solset := solset union {c[i]=0} end_if;
        #assign(solset);#
        userinfo(10, "Time: ", time()-t0, "msec");
        return( subs(_plus(c[j]*diff(z,x$j) $hold(j)=0..i), solset))
      end_if;
      i := i+1;
      userinfo(5, "order is now: ", i);
      Y := expand(subs(diff(Y, x), dxn=term));
      c[i] := genident("c");
      ansatz :=  ansatz + c[i]*Y 
    until FALSE end_repeat;
  end_proc):

